@@ -8,7 +8,8 @@
 #include <ostream>
 #include <istream>
 #include <stdexcept>
-#include <panda/iterator.h>
+#include <functional>
+#include <panda/lib/lib.h>
 
 namespace panda {
 
@@ -16,8 +17,8 @@ using std::size_t;
 
 class string {
 public:
-    typedef base_iterator<char*>                  iterator;
-    typedef base_iterator<const char*>            const_iterator;
+    typedef char*                                 iterator;
+    typedef const char*                           const_iterator;
     typedef std::reverse_iterator<iterator>       reverse_iterator;
     typedef std::reverse_iterator<const_iterator> const_reverse_iterator;
 
@@ -31,6 +32,7 @@ public:
     string (const string& s)                                : _capacity(0) { assign(s); }
     string (const string& s, size_t pos, size_t len = npos) : _capacity(0) { assign(s, pos, len); }
     explicit string (size_t n)                              : _capacity(0), _length(0) { reserve(n); }
+    string (string&& s)                                     : _capacity(0), _length(0) { _u.ptr = ""; swap(s); }
 
     size_t      size     () const { return _length; }
     size_t      length   () const { return _length; }
@@ -40,13 +42,13 @@ public:
     const char* c_str    () const { return _u.ptr; }
     char*       buf      ()       { retain(); return _u.buf; }
 
-    iterator               begin   ()       { iterator it(buf()); return it; }
-    iterator               end     ()       { return iterator(buf() + _length); }
+    iterator               begin   ()       { return buf(); }
+    iterator               end     ()       { return buf() + _length; }
     reverse_iterator       rbegin  ()       { return reverse_iterator(end()); }
     reverse_iterator       rend    ()       { return reverse_iterator(begin()); }
-    const_iterator         cbegin  () const { return const_iterator(data()); }
+    const_iterator         cbegin  () const { return data(); }
     const_iterator         begin   () const { return cbegin(); }
-    const_iterator         cend    () const { return const_iterator(data() + _length); }
+    const_iterator         cend    () const { return data() + _length; }
     const_iterator         end     () const { return cend(); }
     const_reverse_iterator crbegin () const { return const_reverse_iterator(cend()); }
     const_reverse_iterator rbegin  () const { return crbegin(); }
@@ -131,9 +133,10 @@ public:
         return *this;
     }
 
-    string& operator= (const string& source) { if (this != &source) assign(source); return *this; }
-    string& operator= (const char* ptr)      { return assign(ptr); }
-    string& operator= (char c)               { return assign(1, c); }
+    string& operator= (const string& source)  { if (this != &source) assign(source); return *this; }
+    string& operator= (const char* ptr)       { return assign(ptr); }
+    string& operator= (char c)                { return assign(1, c); }
+    string& operator= (string&& source)       { swap(source); return *this; }
 
     operator bool () const { return _length > 0; }
 
@@ -234,6 +237,11 @@ public:
         return begin() + pos;
     }
 
+    // fix ambiguity between iterator(char*) and size_t
+    string& insert (int pos, size_t n, char c)           { return replace(pos, 0, n, c); }
+    string& insert (long int pos, size_t n, char c)      { return replace(pos, 0, n, c); }
+    string& insert (long long int pos, size_t n, char c) { return replace(pos, 0, n, c); }
+
     iterator erase (iterator p) {
         size_t pos = p - begin();
         erase(pos, 1);
@@ -423,4 +431,22 @@ inline void swap (string& l, string& r) { l.swap(r); }
 inline std::ostream& operator<< (std::ostream& os, const string& str) { return os << str.data(); }
 inline std::istream& operator>> (std::istream& is, string& str)       { return is >> str.buf(); }
 
-};
+}
+
+namespace std {
+
+    template<>
+    struct hash<panda::string> {
+        size_t operator() (const panda::string& s) const {
+            return panda::lib::string_hash(s.data(), s.length());
+        }
+    };
+
+    template<>
+    struct hash<const panda::string> {
+        size_t operator() (const panda::string& s) const {
+            return panda::lib::string_hash(s.data(), s.length());
+        }
+    };
+
+}
